/*     PostOffice, demo application for ORBit2.  */
/*     Copyright (C) 2001  Frank Rehberger <frehberg@cs.tu-berlin.de> */
 
/*     This program is free software; you can redistribute it and/or modify */
/*     it under the terms of the GNU General Public License as published by */
/*     the Free Software Foundation; either version 2 of the License, or */
/*     (at your option) any later version. */

/*     This program is distributed in the hope that it will be useful, */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of */
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/*     GNU General Public License for more details. */

/*     You should have received a copy of the GNU General Public License */
/*     along with this program; if not, write to the Free Software */
/*     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <stdio.h>
#include <stdlib.h>

#include "postoffice.h"

/*   MACROS */
#define RAISED_EXCEPTION(_ev)  ((_ev)->_major != CORBA_NO_EXCEPTION)

#define IF_SYSTEM_EXCEPTION(_ev, op)                      \
if ((_ev)->_major == CORBA_SYSTEM_EXCEPTION)              \
   do { op;} while(0)

#define IF_USER_EXCEPTION(_ev, exception_id, op)          \
if((_ev)->_major == CORBA_USER_EXCEPTION                  \
   && (!(exception_id)                                    \
      || !strncmp (exception_id, CORBA_exception_id (_ev),\
		   strlen (exception_id))))               \
   do {op;} while (0)

#define ELSIF_USER_EXCEPTION(_ev, exception_id, op)       \
else if(((_ev)->_major == CORBA_USER_EXCEPTION)           \
   && (!(exception_id)                                    \
      || !strncmp (exception_id, CORBA_exception_id (_ev),\
		   strlen (exception_id))))               \
   do { op; } while(0)


#define ABORT_IF_EXCEPTION(_ev)                              \
if ((_ev)->_major != CORBA_NO_EXCEPTION) {                   \
  g_error(CORBA_exception_id (_ev));                         \
  CORBA_exception_free (_ev);                                \
  abort();                                                   \
}

#define IGNORE_IF_EXCEPTION(_ev)                             \
if ((_ev)->_major != CORBA_NO_EXCEPTION) {                   \
  CORBA_exception_free (_ev);                                \
}

/*   CONST */ 
#define PO_MAX_SEND 50

/*   VAR   */ 
CORBA_ORB         global_orb;  // accessible from signal handler

PostOffice_Letter letter1 = {"goldfinger@gnome.org",             // sender
			     "bond@gnome.org",                   // recipient
			     "what is your name?"};              // content

PostOffice_Letter letter2 = {"bond@gnome.org",                   // sender
			     "goldfinger@gnome.org",             // recipient
			     "My Name is Bond, James Bond!"};    // content

PostOffice_Letter letter3 = {"goldfinger@gnome.org",             // sender
			     "pussy@gnome.org",                  // recipient
			     "Please, show Mr. Bond his room."}; // content


/** reads an IOR-string from file
 *  @arg file containing IOR as character stream
 *  
 */
static 
gchar* 
import_objref (FILE* file)
{
   gchar *buf=NULL;
   
   fscanf (file, "%as", &buf);

   return buf;
}


static 
void
send_letter (PostOffice_Counter po, 
	     PostOffice_Letter *letter,
	     CORBA_Environment *ev)
{
   PostOffice_Counter_send_letter (po, letter, ev);
   if (RAISED_EXCEPTION(ev))
   {
      IF_SYSTEM_EXCEPTION (ev, ABORT_IF_EXCEPTION (ev));
      /* handles for specified user exceptions */
      ELSIF_USER_EXCEPTION (ev, ex_PostOffice_UnknownSender,
			    CORBA_exception_free (ev));
      ELSIF_USER_EXCEPTION (ev, ex_PostOffice_UnknownRecipient,
			    CORBA_exception_free (ev));
      ELSIF_USER_EXCEPTION (ev, ex_PostOffice_MalformedContent,
			    CORBA_exception_free (ev));
      /* abort if any other exception */
      ELSIF_USER_EXCEPTION (ev, NULL, ABORT_IF_EXCEPTION (ev));
   }	 
   g_print ("+ ");
}

static
gboolean
has_letter (PostOffice_Counter po, 
	    char *recipient,
	    CORBA_Environment *ev)
{
   CORBA_boolean has
      =PostOffice_Counter_has_letter_for (po, recipient, ev);
   if (RAISED_EXCEPTION(ev))
   { 
      /* handles for specified user exceptions */
      IF_SYSTEM_EXCEPTION (ev, ABORT_IF_EXCEPTION (ev));
      /* abort if any other exception */
      ELSIF_USER_EXCEPTION (ev, NULL, ABORT_IF_EXCEPTION (ev));
   }	 
   return has;
}

static 
void
fetch_letter (PostOffice_Counter po, 
	      char *recipient,
	      CORBA_Environment *ev)
{
   PostOffice_Letter *letter 
      = PostOffice_Counter_fetch_letter_for (po, recipient, ev);
   if (RAISED_EXCEPTION(ev))
   {
      g_print ("! ");
      IF_SYSTEM_EXCEPTION (ev, ABORT_IF_EXCEPTION (ev));
      /* handles for specified user exceptions */
      ELSIF_USER_EXCEPTION (ev, ex_PostOffice_UnknownRecipient, 
			    CORBA_exception_free (ev));
      /* abort if any other exception */
      ELSIF_USER_EXCEPTION (ev, NULL, ABORT_IF_EXCEPTION (ev));
   }	 
   else
   {
      /* -- do something with letter -- */
      /* ------------------------------ */
      
      /* free memory */
      
      PostOffice_Letter__freekids (letter, NULL); 
      CORBA_free (letter);
      g_print ("- ");
   }
}

int
main (int argc, char *argv[])
{
   long i=0;
   gchar              *objref;
   PostOffice_Counter  obj    = CORBA_OBJECT_NIL;

   CORBA_Environment ev[1] = {NULL};
   CORBA_exception_init(ev);

   global_orb = CORBA_ORB_init(&argc, argv, "orbit-local-orb", ev);
   ABORT_IF_EXCEPTION(ev);

   /* read from file */
   objref=import_objref (stdin);
   if (objref==NULL) abort;

   obj = (PostOffice_Counter) CORBA_ORB_string_to_object (global_orb, 
							  objref, ev);   
   ABORT_IF_EXCEPTION(ev);
   
   for (i=0; i<PO_MAX_SEND; ++i) send_letter (obj, &letter1, ev); 
   for (i=0; i<PO_MAX_SEND; ++i) send_letter (obj, &letter2, ev);
   for (i=0; i<PO_MAX_SEND; ++i) send_letter (obj, &letter3, ev);
   
   while (has_letter(obj, letter1.recipient, ev))
      fetch_letter (obj, letter1.recipient, ev);
   
   while (has_letter(obj, letter2.recipient, ev))
      fetch_letter (obj, letter2.recipient, ev);
   
   while (has_letter(obj, letter3.recipient, ev))
      fetch_letter (obj, letter3.recipient, ev);
   
   /* trigger exceptions */
   for (i=0; i<PO_MAX_SEND; ++i)
      fetch_letter (obj, "foo", ev);
   ABORT_IF_EXCEPTION(ev);  /* no exception should reach this point */

   CORBA_Object_release(obj, ev);
   ABORT_IF_EXCEPTION(ev);

   CORBA_ORB_destroy(global_orb, ev);
   ABORT_IF_EXCEPTION(ev);

   g_print ("\n");

   exit(0);
}

