/*     PostOffice, demo application for ORBit2.  */
/*     Copyright (C) 2001  Frank Rehberger <frehberg@cs.tu-berlin.de> */
 
/*     This program is free software; you can redistribute it and/or modify */
/*     it under the terms of the GNU General Public License as published by */
/*     the Free Software Foundation; either version 2 of the License, or */
/*     (at your option) any later version. */

/*     This program is distributed in the hope that it will be useful, */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of */
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/*     GNU General Public License for more details. */

/*     You should have received a copy of the GNU General Public License */
/*     along with this program; if not, write to the Free Software */
/*     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <signal.h>
#include <orbit/orbit.h>
#include <assert.h>
#include "postoffice.h"
#include "mailbox.h"

#include "postoffice-skelimpl.c"

/*   MACROS */
#define WARNING_IF_EXCEPTION(ev) if (ev->_major != CORBA_NO_EXCEPTION)      \
            { g_warning ("caught exception %s", CORBA_exception_id (ev));   \
              CORBA_exception_free (ev); }

#define ABORT_IF_EXCEPTION(ev) if (ev->_major != CORBA_NO_EXCEPTION)        \
            { g_error ("caught exception %s", CORBA_exception_id (ev));     \
              CORBA_exception_free (ev); abort(); }

#define IGNORE_IF_EXCEPTION(ev) if (ev->_major != CORBA_NO_EXCEPTION)       \
            {  CORBA_exception_free (ev); }


/*   VAR   */
static CORBA_ORB  global_orb = CORBA_OBJECT_NIL; /*global declaration of orb*/
static CORBA_Environment ev[1];


/*************************************************************************/
/****                     signal handler                              ****/
/*************************************************************************/


static
void 
signal_handler(int sig)
{
	static CORBA_Environment local_ev[1];
	// if using Linux-kernel 2.4 printing anything to console within
	// sighandler function is prohibited, as this might cause deadlocks
       
	// ignore further signals 
	struct sigaction ignore;
       
	ignore.sa_handler = SIG_IGN;
       
	sigemptyset (&ignore.sa_mask);
       
	ignore.sa_flags = 0;
       
	if (sigaction (SIGINT, &ignore, (struct sigaction *)0) == -1)
		abort ();
       
	if (sigaction (SIGTERM, &ignore, (struct sigaction *)0) == -1)
		abort ();
       
	if (sigaction (SIGHUP, &ignore, (struct sigaction *)0) == -1)
		abort (); 
       
	/*  Terminate event loop  */
       
	// usual code
	if (global_orb != CORBA_OBJECT_NIL)
	{
		CORBA_exception_init(local_ev);

		CORBA_ORB_shutdown (global_orb, FALSE, local_ev);
		IGNORE_IF_EXCEPTION (local_ev); 

		global_orb=CORBA_OBJECT_NIL;
	}
}

/*************************************************************************/
/****                            main                                 ****/
/*************************************************************************/

int
main(int argc, char *argv[])
{
	PostOffice_Counter               servant;
	PortableServer_POA               root_poa;
	PortableServer_POAManager        pm;
	CORBA_char                      *objref = NULL;

	struct sigaction sa;            /* New signal state */

	sa.sa_handler = signal_handler; /* Set handler function */
	sigfillset(&sa.sa_mask);        /* Mask all other signals while handler
					   runs */
	sa.sa_flags =0 |SA_RESTART;     /* Restart interrupted syscalls */
   
	if (sigaction (SIGINT, &sa, (struct sigaction *)0) == -1)
		abort ();
   
	if (sigaction(SIGHUP, &sa, (struct sigaction *)0) == -1)
		abort ();
   
	if (sigaction(SIGTERM, &sa, (struct sigaction *)0) == -1)
		abort();
   
	CORBA_exception_init(ev);

	/* create Object Request Broker (ORB) */

	global_orb = CORBA_ORB_init(&argc, argv, "orbit-local-orb", ev);
	ABORT_IF_EXCEPTION(ev);

	/* create Portable Object Adaptor (POA) */

	root_poa = (PortableServer_POA)
		CORBA_ORB_resolve_initial_references(global_orb,  "RootPOA", ev);
	ABORT_IF_EXCEPTION(ev);

	/* create servant instance(s) */   

	servant = impl_PostOffice_Counter__create (root_poa, ev);
	ABORT_IF_EXCEPTION(ev);

	/* activate POA Manager */

	pm = PortableServer_POA__get_the_POAManager(root_poa, ev);
	ABORT_IF_EXCEPTION(ev);

	PortableServer_POAManager_activate(pm, ev);
	ABORT_IF_EXCEPTION(ev);

	/* write objref to file */

	objref = CORBA_ORB_object_to_string(global_orb, servant, ev);
	ABORT_IF_EXCEPTION(ev);

	/* print ior to terminal */
	g_print ("%s\n", objref);
   
	CORBA_free (objref); 
	objref = NULL;

	/* enter main loop */
 
	CORBA_ORB_run(global_orb, ev);
	ABORT_IF_EXCEPTION(ev);

	/* main loop has been left due to CORBA_ORB_shutdown(.) */

	/* release managed object of servant */
	CORBA_Object_release(servant, ev);
	ABORT_IF_EXCEPTION(ev); 

	/* tear down the ORB */
	if (global_orb != CORBA_OBJECT_NIL)
	{
		/* going to destroy orb.. */
		CORBA_ORB_destroy(global_orb, ev);
		ABORT_IF_EXCEPTION(ev);
	}

	/* return with SUCCESS  */
	exit (0);
}

