#include "postoffice.h"

/*** App-specific servant structures ***/

typedef struct
{
   POA_PostOffice_Counter servant;
   PortableServer_POA poa;

   /* ------ add private attributes here ------ */
   Mailbox*     mailbox;             /* mailbox */
   /* ------ ---------- end ------------ ------ */
}
impl_POA_PostOffice_Counter;

/*** Implementation stub prototypes ***/

static void impl_PostOffice_Counter__destroy(impl_POA_PostOffice_Counter *
					     servant, CORBA_Environment * ev);
static CORBA_boolean
impl_PostOffice_Counter_has_letter_for(impl_POA_PostOffice_Counter * servant,
				       const CORBA_char * recipient,
				       CORBA_Environment * ev);

static PostOffice_Letter
   *impl_PostOffice_Counter_fetch_letter_for(impl_POA_PostOffice_Counter *
					     servant,
					     const CORBA_char * recipient,
					     CORBA_Environment * ev);

static void
impl_PostOffice_Counter_send_letter(impl_POA_PostOffice_Counter * servant,
				    const PostOffice_Letter * letter,
				    CORBA_Environment * ev);

/*** epv structures ***/

static PortableServer_ServantBase__epv impl_PostOffice_Counter_base_epv = {
   NULL,			/* _private data */
   (gpointer) & impl_PostOffice_Counter__destroy,	/* finalize routine */
   NULL,			/* default_POA routine */
};
static POA_PostOffice_Counter__epv impl_PostOffice_Counter_epv = {
   NULL,			/* _private */
   (gpointer) & impl_PostOffice_Counter_has_letter_for,

   (gpointer) & impl_PostOffice_Counter_fetch_letter_for,

   (gpointer) & impl_PostOffice_Counter_send_letter,

};

/*** vepv structures ***/

static POA_PostOffice_Counter__vepv impl_PostOffice_Counter_vepv = {
   &impl_PostOffice_Counter_base_epv,
   &impl_PostOffice_Counter_epv,
};

/*** Stub implementations ***/

static PostOffice_Counter
impl_PostOffice_Counter__create(PortableServer_POA poa,
				CORBA_Environment * ev)
{
   PostOffice_Counter retval;
   impl_POA_PostOffice_Counter *newservant;
   PortableServer_ObjectId *objid;

   newservant = g_new0(impl_POA_PostOffice_Counter, 1);
   newservant->servant.vepv = &impl_PostOffice_Counter_vepv;
   newservant->poa = poa;
   POA_PostOffice_Counter__init((PortableServer_Servant) newservant, ev);
   /* Before servant is going to be activated all
    * private attributes must be initialized.  */

   /* ------ init private attributes here ------ */
   newservant->mailbox = mailbox_new();
   if (newservant->mailbox==NULL) 
   {
      /* creation of mailbox failed - finalize servant structures */ 
      POA_PostOffice_Counter__fini((PortableServer_Servant) newservant, ev);
      g_free(newservant);
      return CORBA_OBJECT_NIL;
   }
   /* ------ ---------- end ------------- ------ */

   objid = PortableServer_POA_activate_object(poa, newservant, ev);
   CORBA_free(objid);
   retval = PortableServer_POA_servant_to_reference(poa, newservant, ev);

   return retval;
}

static void
impl_PostOffice_Counter__destroy(impl_POA_PostOffice_Counter * servant,
				 CORBA_Environment * ev)
{
   /* No further remote method calls are delegated to 
    * servant; you may free your private attributes now. */

   /* ------ free private attributes here ------ */
   mailbox_free (servant->mailbox);
   servant->mailbox = NULL;
   /* ------ ---------- end ------------- ------ */

   POA_PostOffice_Counter__fini((PortableServer_Servant) servant, ev);
}

static CORBA_boolean
impl_PostOffice_Counter_has_letter_for(impl_POA_PostOffice_Counter * servant,
				       const CORBA_char * recipient,
				       CORBA_Environment * ev)
{
   CORBA_boolean retval;

   /* ------   insert method code here   ------ */
   retval = mailbox_has_letter_for (servant->mailbox, recipient);
   /* ------ ---------- end ------------ ------ */

   return retval;
}

static
PostOffice_Letter*
to_corba_letter (const Letter* src)
{
   PostOffice_Letter* dest = PostOffice_Letter__alloc ();

   dest->sender    = CORBA_string_dup (src->sender);
   dest->recipient = CORBA_string_dup (src->recipient);
   dest->content   = CORBA_string_dup (src->content);

   return dest;
}

static
Letter*
to_internal_letter (const  PostOffice_Letter* src)
{
   Letter* dest = g_new0 (Letter, 1);

   dest->sender    = g_strdup (src->sender);
   dest->recipient = g_strdup (src->recipient);
   dest->content   = g_strdup (src->content);

   return dest;
}

static
void
internal_letter_free (Letter* letter)
{
   g_free (letter->sender);
   g_free (letter->recipient);
   g_free (letter->content);
   g_free (letter);
}

static PostOffice_Letter *
impl_PostOffice_Counter_fetch_letter_for(impl_POA_PostOffice_Counter *
					 servant,
					 const CORBA_char * recipient,
					 CORBA_Environment * ev)
{
   PostOffice_Letter *retval;

   /* ------   insert method code here   ------ */
   {
      Letter* letter = mailbox_fetch_letter_for (servant->mailbox, recipient);
      
      if (letter!=NULL)
      {
	 PostOffice_Letter* corba_letter = to_corba_letter (letter);
	 internal_letter_free (letter);
	 retval = corba_letter;
      }
      else
      {
	 /* throw exception */
	 PostOffice_UnknownRecipient* exdata
	    = PostOffice_UnknownRecipient__alloc();
	 exdata->message = CORBA_string_dup ("Unknown recipient");
	 
	 CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
			      ex_PostOffice_UnknownRecipient,
			      exdata);
	 retval = NULL;
      }
      
   }
   /* ------ ---------- end ------------ ------ */

   return retval;
}

static void
impl_PostOffice_Counter_send_letter(impl_POA_PostOffice_Counter * servant,
				    const PostOffice_Letter * letter,
				    CORBA_Environment * ev)
{
   /* ------   insert method code here   ------ */
   {
      Letter* internal_letter = to_internal_letter (letter);
      
      mailbox_send_letter (servant->mailbox, internal_letter);
      
      internal_letter_free (internal_letter);
   }
   /* ------ ---------- end ------------ ------ */
}
